@@ -154,7 +154,7 @@ type Rec e = RecordMap Ident (Range, e)
 newtype Program name = Program [TopDecl name]
                        deriving (Show)
 
-{- | A module for the pre-typechecker phasese. The two parameters are:
+{- | A module for the pre-typechecker phases. The two parameters are:
 
   * @mname@ the type of module names. This is because top-level and nested
     modules use different types to identify a module.
@@ -176,7 +176,7 @@ data ModuleG mname name = Module
   } deriving (Show, Generic, NFData)
 
 
--- | Different flavours of module we have.
+-- | Different flavours of modules we have.
 data ModuleDefinition name =
     NormalModule [TopDecl name]
 
@@ -587,12 +587,14 @@ data Newtype name = Newtype
   , nParams   :: [TParam name]       -- ^ Type params
   , nConName  :: !name               -- ^ Constructor function name
   , nBody     :: Rec (Type name)     -- ^ Body
+  , nDeriving :: [Located name]      -- ^ Derived typeclasses
   } deriving (Eq, Show, Generic, NFData)
 
 data EnumDecl name = EnumDecl
   { eName     :: Located name               -- ^ Type name
   , eParams   :: [TParam name]              -- ^ Type params
   , eCons     :: [TopLevel (EnumCon name)]  -- ^ Value constructors
+  , eDeriving :: [Located name]             -- ^ Derived typeclasses
   } deriving (Show, Generic, NFData)
 
 data EnumCon name = EnumCon
@@ -1631,12 +1633,14 @@ instance NoPos (Newtype name) where
                     , nParams   = nParams n
                     , nConName  = nConName n
                     , nBody     = fmap noPos (nBody n)
+                    , nDeriving = map noPos (nDeriving n)
                     }
 
 instance NoPos (EnumDecl name) where
   noPos n = EnumDecl { eName    = noPos (eName n)
                      , eParams  = eParams n
                      , eCons    = map noPos (eCons n)
+                     , eDeriving = map noPos (eDeriving n)
                      }
 
 instance NoPos (EnumCon name) where
