@@ -10,7 +10,7 @@ import qualified Data.List.NonEmpty as NE
 import qualified Data.Set as Set
 import Control.DeepSeq(NFData)
 import GHC.Generics(Generic)
-import Data.List((\\),sortBy,partition)
+import Data.List((\\),sortBy,partition,intersperse)
 import Data.Function(on)
 
 import Cryptol.Utils.Ident(Ident,Namespace(..))
@@ -89,6 +89,7 @@ data Warning  = DefaultingKind (P.TParam Name) P.Kind
               | DefaultingWildType P.Kind
               | DefaultingTo !TVarInfo Type
               | NonExhaustivePropGuards Name
+              | DuplicateDeriving PC
                 deriving (Show, Generic, NFData)
 
 -- | Various errors that might happen during type checking/inference
@@ -117,6 +118,19 @@ data Error    = KindMismatch (Maybe TypeSource) Kind Kind
                 --   More patterns provided for a bind than expected,
                 --   given its signature.
 
+              | ClassNotDerivable
+                -- ^ Deriving unsupported typeclass
+                  Name   -- ^ The thing that the user tried to derive
+                  String -- ^ Description of the type declaration
+                  [PC]   -- ^ The supported typeclasses for deriving on this
+                         --   sort of type declaration
+
+              | DerivingMissingSuperclasses
+                -- ^ Deriving instance for a class without deriving instances
+                --   for its superclasses
+                  PC   -- ^ The class listed in the deriving clause
+                  [PC] -- ^ Its superclasses not listed in the deriving clause
+
               | TypeMismatch TypeSource Path Type Type
                 -- ^ Expected type, inferred type
 
@@ -161,9 +175,9 @@ data Error    = KindMismatch (Maybe TypeSource) Kind Kind
                 -- ^ Too many positional type arguments, in an explicit
                 -- type instantiation
 
-              | BadParameterKind TParam Kind
-                -- ^ Kind other than `*` or `#` given to parameter of
-                --   type synonym, newtype, function signature, etc.
+              | BadParameterKind TParam Kind [Kind]
+                -- ^ We expected one of the kinds in the list, but got
+                -- the given one instead.
 
               | CannotMixPositionalAndNamedTypeParams
 
@@ -245,6 +259,8 @@ errorImportance err =
     KindMismatch {}                                  -> 10
     TyVarWithParams {}                               -> 9
     TooManyParams{}                                  -> 9
+    ClassNotDerivable{}                              -> 8
+    DerivingMissingSuperclasses{}                    -> 8
     TypeMismatch {}                                  -> 8
     EnumTypeMismatch {}                              -> 7
     SchemaMismatch {}                                -> 7
@@ -301,6 +317,7 @@ instance TVars Warning where
       DefaultingWildType {} -> warn
       DefaultingTo d ty     -> DefaultingTo d $! (apSubst su ty)
       NonExhaustivePropGuards {} -> warn
+      DuplicateDeriving {}  -> warn
 
 instance FVS Warning where
   fvs warn =
@@ -309,6 +326,7 @@ instance FVS Warning where
       DefaultingWildType {} -> Set.empty
       DefaultingTo _ ty     -> fvs ty
       NonExhaustivePropGuards {} -> Set.empty
+      DuplicateDeriving {}  -> Set.empty
 
 instance TVars Error where
   apSubst su err =
@@ -322,6 +340,8 @@ instance TVars Error where
       SchemaMismatch i t1 t2  ->
         SchemaMismatch i !$ (apSubst su t1) !$ (apSubst su t2)
       TooManyParams b t i j     -> TooManyParams b !$ (apSubst su t) .$ i .$ j
+      ClassNotDerivable {}      -> err
+      DerivingMissingSuperclasses {} -> err
       TypeMismatch src pa t1 t2 -> TypeMismatch src pa !$ (apSubst su t1) !$ (apSubst su t2)
       EnumTypeMismatch t        -> EnumTypeMismatch !$ apSubst su t
       InvalidConPat {}          -> err
@@ -375,6 +395,8 @@ instance FVS Error where
       RecursiveTypeDecls {}     -> Set.empty
       SchemaMismatch _ t1 t2    -> fvs (t1,t2)
       TooManyParams _ t _ _     -> fvs t
+      ClassNotDerivable {}      -> Set.empty
+      DerivingMissingSuperclasses {} -> Set.empty
       TypeMismatch _ _ t1 t2    -> fvs (t1,t2)
       EnumTypeMismatch t        -> fvs t
       InvalidConPat {}          -> Set.empty
@@ -393,7 +415,7 @@ instance FVS Error where
       UndefinedTypeParameter {}             -> Set.empty
       RepeatedTypeParameter {}              -> Set.empty
       AmbiguousSize _ t -> fvs t
-      BadParameterKind tp _ -> Set.singleton (TVBound tp)
+      BadParameterKind tp _ _ -> Set.singleton (TVBound tp)
 
       BareTypeApp -> Set.empty
 
@@ -440,6 +462,9 @@ instance PP (WithNames Warning) where
         text "Could not prove that the constraint guards used in defining" <+>
         pp n <+> text "were exhaustive."
 
+      DuplicateDeriving pc ->
+        "Duplicate" <+> pp pc <+> "in deriving clause"
+
 instance PP (WithNames Error) where
   ppPrec _ (WithNames err names) =
     case err of
@@ -506,6 +531,16 @@ instance PP (WithNames Error) where
             , "Actual number of parameters:" <+> int i
             , "When defining" <+> quotes ((pp n <> ":") <+> ppWithNames names t) ]
 
+      ClassNotDerivable badCls declDesc allowed ->
+        "Cannot derive" <+> backticks (pp badCls) $$
+        nested ("Derivable constraints for" <+> text declDesc <+> "declarations are:")
+          (commaSepFill (map pp allowed))
+
+      DerivingMissingSuperclasses pc supers ->
+        "Missing superclass(es) for" <+> pp pc $$
+        "To derive this constraint, you must also derive:" <+>
+          commaSepFill (map pp supers)
+
       TypeMismatch src pa t1 t2 ->
         addTVarsDescsAfter names err $
         nested "Type mismatch:" $
@@ -600,10 +635,11 @@ instance PP (WithNames Error) where
                    ++ [ "When checking" <+> pp src ]
                )
 
-      BadParameterKind tp k ->
+      BadParameterKind tp k ks ->
         addTVarsDescsAfter names err $
         vcat [ "Illegal kind assigned to type variable:" <+> ppWithNames names tp
-             , "Unexpected:" <+> pp k
+             , "Expected:" <+> hsep (intersperse "or" (map pp ks))
+             , "Actual:" <+> pp k
              ]
 
       TooManyPositionalTypeParams ->
