@@ -39,7 +39,7 @@ import           MonadLib hiding (mapM)
 
 import           Cryptol.ModuleSystem.Name
                     (FreshM(..),Supply,mkLocal,asLocal
-                    , nameInfo, NameInfo(..),NameSource(..), nameTopModule)
+                    , nameInfo, NameInfo(..),NameSource(..),nameTopModule)
 import           Cryptol.ModuleSystem.NamingEnv.Types
 import qualified Cryptol.ModuleSystem.Interface as If
 import           Cryptol.Parser.Position
@@ -633,13 +633,13 @@ checkParamKind tp flav k =
         KNum  -> return ()
         KType -> return ()
         KProp -> return ()
-        _ -> recordError (BadParameterKind tp k)
+        _ -> recordError (BadParameterKind tp k [KNum, KType, KProp])
 
     starOrHash =
       case k of
         KNum  -> return ()
         KType -> return ()
-        _ -> recordError (BadParameterKind tp k)
+        _ -> recordError (BadParameterKind tp k [KNum,KType])
 
 -- | Generate a new free type variable.
 newTParam :: P.TParam Name -> TPFlavor -> Kind -> InferM TParam
